
#include <QProcess>
#include <QGraphicsScene>
#include <QGraphicsLinearLayout>
#include <QMessageBox>
#include <QDBusMessage>
#include <QDBusConnection>

#include <KStandardDirs>
#include <KLocale>
#include <KToolInvocation>
#include <KFilePlacesModel>
#include <KFileItem>
#include <konq_operations.h>

#include <Plasma/ToolTipManager>

#include "places.h"

using namespace Plasma;

Places::Places(QObject *parent, const QVariantList &args)
    : PopupApplet(parent, args)
{
    setBackgroundHints(DefaultBackground);
    setAspectRatioMode(IgnoreAspectRatio);
    setHasConfigurationInterface(false);
    KGlobal::locale()->insertCatalog("libkonq");
    KGlobal::locale()->insertCatalog("dolphin");
}


Places::~Places()
{
}

void Places::init()
{
    KFilePlacesModel *placemodel = new KFilePlacesModel;

    place = new KFilePlacesView;
    place->setAttribute(Qt::WA_NoSystemBackground);
    place->setDropOnPlaceEnabled(true);
    place->setModel(placemodel);
    place->clearSelection();

    m_pop = new QGraphicsProxyWidget(this);
    m_pop->setMinimumSize(300, (placemodel->rowCount() - placemodel->hiddenCount() + 2)*42);

    QGraphicsLinearLayout *vl = new QGraphicsLinearLayout(Qt::Vertical, m_pop);
    vl->addItem(scene()->addWidget(place));

    setGraphicsWidget(m_pop);
    setPopupIcon("user-home");

    Plasma::ToolTipManager::self()->setContent(this, Plasma::ToolTipContent(i18nc("@title:window", "Places"), "", KIcon("user-home")));

    connect(place, SIGNAL(urlChanged(const KUrl&)), this, SLOT(openUrl(const KUrl&)));
    connect(place, SIGNAL(urlsDropped(const KUrl&, QDropEvent*, QWidget*)),
            this, SLOT(catchUrl(const KUrl&, QDropEvent*, QWidget*)));
}

void Places::openUrl(const KUrl &url)
{
    QString program(KStandardDirs::findExe("kioclient"));
    QString strurl(url.pathOrUrl());
    if( strurl.startsWith("http:/") || strurl.startsWith("https:/") || program.isEmpty() )
    {
	KToolInvocation::invokeBrowser(url.pathOrUrl());
    }
    else
    {
	QProcess::startDetached(program, QStringList() << "exec" << strurl);
    }
}

void Places::dragEnterEvent(QGraphicsSceneDragDropEvent*)
{
    place->clearSelection();
    showPopup();
}

void Places::catchUrl(const KUrl& destination, QDropEvent* event, QWidget* widget)
{
    const QMimeData* mimeData = event->mimeData();
    if (mimeData->hasFormat("application/x-kde-dndextract")) {
        QString remoteDBusClient = mimeData->data("application/x-kde-dndextract");
        QDBusMessage message = QDBusMessage::createMethodCall(remoteDBusClient, "/DndExtract",
                                                              "org.kde.DndExtract", "extractSelectedFilesTo");
        message.setArguments(QVariantList() << destination.path());
        QDBusConnection::sessionBus().call(message);
    } else {
        const KUrl::List urls = KUrl::List::fromMimeData(event->mimeData());
        const int urlsCount = urls.count();
        if (urlsCount == 0) {
            // TODO: handle dropping of other data
        } else if ((urlsCount == 1) && (urls.first() == destination)) {
            QMessageBox::critical(0, "Error", i18nc("@info:status", "A folder cannot be dropped into itself"));
        } else {
            KonqOperations::doDrop(KFileItem(), destination, event, widget);
        }
    }
}

#include "places.moc"
